﻿<# Create the connection to the SCOM by importing the module #>
param($nfspath, $serviceTagFolders)
#$nfspath = "C:\nfsshare\CMC"
$logDirectory = "DellSDKCMC_log"
$ErrorActionPreference = "SilentlyContinue"
$OMPSInstallKey = "HKLM:\Software\Microsoft\System Center Operations Manager\12\Setup\Powershell\V2"
$regKey = get-item $OMPSInstallKey
$OMPSInstallPath = $regKey.GetValue("InstallDirectory")
$omModule = $OMPSInstallPath + "OperationsManager"
Import-Module -Name $omModule
$fqdn = [System.Net.Dns]::GetHostByName(($env:computerName))
$computerName = $fqdn.HostName
$mg = New-Object -TypeName "Microsoft.EnterpriseManagement.ManagementGroup" -ArgumentList @($computerName)
$discoveryData = new-Object -TypeName "Microsoft.EnterpriseManagement.ConnectorFramework.IncrementalDiscoveryData"
$Connector = Get-SCOMConnector -Name "Dell Device Monitoring Connector"

if($Connector -eq $null)
{
    $connectorInst = new-Object -TypeName 'Microsoft.EnterpriseManagement.ConnectorFramework.ConnectorInfo'
    $connectorInst.Description = "Connector for Monitoring Dell Devices"
    $connectorInst.Name = "Dell Device Monitoring Connector"
    $connectorInst.DisplayName = "Powershell Inbound connector";

    $ConnectorGuid = [System.Guid]::NewGuid().Guid
    $cfa = $mg.GetConnectorFrameworkAdministration()
    $Connector = $cfa.Setup($connectorInst, $ConnectorGuid)
}

function ConvertPSObjectToHashtable{
    param (
        [Parameter(ValueFromPipeline)]
        $InputObject
    )

    process
    {
        if ($null -eq $InputObject)
        { 
            return $null
        }

        if ($InputObject -is [System.Collections.IEnumerable] -and $InputObject -isnot [string])
        {
            $collection = @(
                foreach ($object in $InputObject) { ConvertPSObjectToHashtable $object }
            )
            Write-Output -NoEnumerate $collection
        }
        elseif ($InputObject -is [psobject])
        {
            $hash = @{}

            foreach ($property in $InputObject.PSObject.Properties)
            {
                $hash[$property.Name] = ConvertPSObjectToHashtable $property.Value
            }
            $hash
        }
        else
        {
            $InputObject
        }
    }
} 

<# Creates the log file to capture the logs #>
Function CreateLogFile
{
    param($logFileName)
    try{
	    $TempFolder = $env:SystemRoot + "\Temp"
	    $LogLocation = $TempFolder + "\" + $logDirectory + "\"
	    If(!(Test-Path -path($TempFolder))){
		    # create the directory if not present
		    New-Item $TempFolder -type directory 
	    }
	    If(!(Test-Path -path($LogLocation))){
		    # create the directory if not present
		    New-Item $LogLocation -type directory 
	    }
	    $Global:LogFileLocation = $LogLocation + $logFileName + ".log"
	    If(!(Test-Path -path($LogFileLocation))){
		    # create the file if it does not exist
		    New-Item $LogFileLocation -type file
	    }
	    Else{
		    $logFileSize = Get-ChildItem $LogFileLocation | ForEach-Object {($_.Length/1KB)}
		    If ($logFileSize -gt 512){
			    # existingLogFile is greater than 512 KB
			    $archiveTime = Get-Date -f "yyyy-MM-dd_HH.mm" 
			    Rename-Item $LogFileLocation ("ArchivedLog_" + $logFileName + "_" + $archiveTime + ".log")
			    New-Item $LogFileLocation ChildItem-type file
		    }                    
	    }
    }
    catch{
	    $ErrorMessage = $_.Exception.Message
	    $FailedItem = $_.Exception.ItemName
	    $message = "Exception in creating log file. Exception-Message:" + $ErrorMessage + $FailedItem
	    #$api.LogScriptEvent("DellSDKCMCDiscovery.ps1",1345,0,$message)
    }
}


<# Function to write the log to the log file #>
Function psDebugLog
{		
	param($level, $message)
	if (($level -gt 0))
    {
	   $currentTime = Get-Date -f "yyyy-MM-dd_HH.mm.ss"
	   Out-File -FilePath $LogFileLocation -InputObject ($currentTime + " :: " + $message) -Append
	}
}

<# Function to the get the scom class #>
Function GetMonitoringClass
{
    param($name)
    $getscomClass= Get-SCOMClass -Name $name
    $retClass = $mg.GetMonitoringClass($getscomClass.Id)
    return $retClass
}

<# Function to get the SCOM class relation #>
Function GetMonitoringRelation
{
    param($name)
    $getscomRelation= Get-SCOMRelationship -Name $name
    $retRelation = $mg.GetMonitoringRelationshipClass($getscomRelation.Id)
    return $retRelation
}

<# Function which creats the containment relation between the source and the target #>
Function CreateContainmentRelation
{
    param($relation, $sourceInst, $targetInst)
    $containmentRelationship = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringRelationshipObject($relation)
    $containmentRelationship.SetSource($sourceInst)
    $containmentRelationship.SetTarget($targetInst)
    $discoveryData.Add($containmentRelationship)
}

<# Fuction provides the source and the target to for createcontainment relation function #>
Function CallCreateContainment
{
    param($source, $target, $relation)
    if($source.MonitoringClass -eq $null)
    {
        $sourceClass = GetMonitoringClass $source
        $sourceInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($sourceClass)
    }
    else
    {
        $sourceInst = $source
    }
    if($target.MonitoringClass -eq $null)
    {
        $targetClass = GetMonitoringClass $target
        $targetInst = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($targetClass) 
    }
    else
    {
        $targetInst = $target
    }
    $relation = GetMonitoringRelation $relation
    CreateContainmentRelation $relation $sourceInst $targetInst
}


<#
    This function will create containment relationship between VirtualDisk instances and Physicaldisk instances
    Parameter - $virtualDiskInstances = HashMap containing Virtualdisk instances
                $physicalDiskInstances = HashMap contaning physicaldisk instances with key as virtualdisk id
#>
Function CreateContainmentForVirtualPhysical
{
    param($virtualDiskInstances,$physicalDiskInstances)
    $pdInstance
    $vdInstance
    foreach($key in $physicalDiskInstances.Keys){
         $vdInstance = $virtualDiskInstances.$key
         foreach($pdInstance in $physicalDiskInstances.$key){
             $relationName = "VirtualDisk.Contains.PhysicalDisk"
             $virtualPhysicalRelationID = GetMonitoringRelation $relationName
             CreateContainmentRelation $virtualPhysicalRelationID $vdInstance $pdInstance
         }
    }
}

<# Hash Map which conatins the class and their relationships in key and value pair #>
$hash_CMCGroupMap = [ordered]@{"DellHWGroup.Contains.CMCGroup" = @("Dell.Base.HardwareGroup", "Dell.SDK.CMCGroup");
                        "DellSDKCMCGroup.Contains.M1000EGroup" = @("Dell.SDK.CMCGroup", "Dell.SDK.M1000EGroup");
                        "DellSDKCMCGroup.Contains.VRTXGroup" = @("Dell.SDK.CMCGroup", "Dell.SDK.VRTXGroup");
                        "DellSDKCMCGroup.Contains.FX2Group"= @("Dell.SDK.CMCGroup","Dell.SDK.FX2Group");
                        "DellSDKCMCGroup.Contains.MX7000Group"= @("Dell.SDK.CMCGroup","Dell.SDK.MX7000Group")
                        }
$rootInstance = $null
$computerClassID = GetMonitoringClass "Microsoft.Windows.Computer"
$entityClass = GetMonitoringClass -Name "System.Entity"

<# Getting SCOM Class to different different types of CMC Chassis #>
$M1000ECMCClass = GetMonitoringClass "Dell.SDK.M1000E"
$VRTXCMCClass = GetMonitoringClass "Dell.SDK.VRTX"
$FX2CMCClass = GetMonitoringClass "Dell.SDK.FX2"
$MX7000Class = GetMonitoringClass "Dell.SDK.MX7000"

<# Creates Containment relation with Dell Hardware Group #>
Foreach ($key in $hash_CMCGroupMap.keys)
{
    CallCreateContainment $hash_CMCGroupMap.Item($key)[0] $hash_CMCGroupMap.Item($key)[1] $key
}


function cleanUpinstances{
    $deletedInsts = @{}
    if( $script:jsondeleteinst._deleted_comps)
    {
        $deletedInsts = $deletedInsts +  $script:jsondeleteinst._deleted_comps
    }
    if( $script:jsondeleteinst._deleted_instances)
    {
        $deletedInsts =  $deletedInsts + $script:jsondeleteinst._deleted_instances
        
    }
    
    if($deletedInsts)
    {
        foreach($comp in $deletedInsts.getEnumerator())
        {
            $classname = "Dell.SDK.CMC."+$comp.Name
            if($comp.Name -eq "StorageModule")
            {
                $classname = "Dell.SDK.CMC."+ "ComputeModule"
            }

            $objsTobeDel = Get-SCOMClass -Name $className | Get-SCOMClassInstance
            foreach($jsobj in $comp.Value)
            {
                $jkey = $jsobj.Key
                foreach($obj in $objsTobeDel)
                {
                    $instancekey = $obj.GetMonitoringPropertyValue(($obj.GetMonitoringProperties() | where {$_.Name -eq ‘key’}))
                    if(($obj.FullName -match $rootKey) -and ($instancekey -eq $jkey))
                    {
                        #Write-Host "DELETING "$obj.FullName -ForegroundColor DarkRed
                        $discoveryData.remove($obj)
                    }
                }
            }
        }
    }
}

<#This function will create subsystems node i.e It will 
a) create SCom-class instance for subSystem and
b) create relationships
Parameters - $rootParent = Inventory.Json reference
$rootParentName = CompTree.json reference#>
Function CreateSubsystemNodes{
    param($rootParent,$rootParentName, $Device)
    foreach($subSystem in $rootParent.PsObject.Properties){ 
        if((!($skip_Components -contains $subSystem.Name))){
            if($array_SkipSubsystem.Contains($rootParentName)){
                $subSystemClassName = "Dell.SDK." + $Device +"." +$rootParentName
            } 
            else{
                $subSystemClassName = "Dell.SDK." + $Device +"." +$subSystem.Name +".Subsystem"
            }

            if($subSystem.Name -eq "StorageModule")
            {
            
            $subSystemClassName = "Dell.SDK." + $Device +"." +"ComputeModule" +".Subsystem"

            }
            if(!($child_ParentMAP.Contains($subSystem.Name))){
                $child_ParentMAP.Add($subSystem.Name,@($child_ParentMAP.Item($rootParentName)))
            }
            else{
                $child_ParentMAP.Item($subSystem.Name) = $child_ParentMAP.Item($subSystem.Name) + @($child_ParentMAP.Item($rootParentName))
            }
            
        
            if(!$array_SkipSubsystem.Contains($subSystem.Name)){
                $monitoringClass = GetMonitoringClass $subSystemClassName 
                $classInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
                
                if($component_ContainmentRelation.Contains($subSystem.Name)){
                     $child_ParentMAP.Item($subSystem.Name) = @{"Microsoft.Windows.Computer" = $targetComputer}

                }
                $parentkeymap = $child_ParentMAP.Item($subSystem.Name)
                
                foreach($parent in $parentkeymap.keys){
                    if(!$array_SkipSubsystem.Contains($parentkeymap.$parent)){
                        $parentClass = GetMonitoringClass $parent  
                        if($parent -eq "Microsoft.Windows.Computer"){
                            $classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty("PrincipalName"),$parentkeymap.$parent)  
                        }
                        else{
							$classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty("Key"),$parentkeymap.$parent)
						}
                    }
                }
                if($monitoringClass.PropertyCollection.Name.Contains("Component")){
                     $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Component"),"SubSystem")
                }

                if($monitoringClass.PropertyCollection.Name.Contains("RemoteAccessIP")){
                     $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("RemoteAccessIP"),$IPAddress)
                }

                if($component_ContainmentRelation.Contains($subSystem.Name)){
                    $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
                }else{
                    $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$subSystem.Name)
                }

                If($subSystem.Name -eq "Storage"){
                    $classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), "Chassis Storage")
                }elseif (!$array_SkipSubsystem.Contains($subSystem.Name)){
                    $classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $subSystem.Name + " Group")
                }else{
                    $classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $subSystem.Name)
                }
                $discoveryData.Add($classInstance)
            }
            if(($component_ContainmentRelation.Contains($subSystem.Name))){
                    $relationName = "CMC.Contains." + $subSystem.Name + "Subsystem"
                    $storageServerRelationID = GetMonitoringRelation $relationName
                    CreateContainmentRelation $storageServerRelationID $rootInstance $classInstance
            }
            if($subSystem.Value -is [system.array]){        
               foreach($leaf in $subSystem.Value){    
                    if($component_ContainmentRelation.Contains($subSystem.Name)){
                         $child_ParentMAP.Add($leaf,@(@{$subSystemClassName = $rootkey })+ $child_ParentMAP.Item($subSystem.Name))
                    }
                    elseif($rootParentName.Contains("Virtual") -and ($subSystem.Name -eq "PhysicalDisk")){
                        if($child_ParentMAP.ContainsKey($leaf)){
                            $child_ParentMAP.Item($leaf) = $child_ParentMAP.Item($leaf) + @{"Dell.SDK.CMC.VirtualDisk" = $rootParentName} + @{"Dell.SDK.CMC.VirtualDisk.Subsystem" = "VirtualDisk"}
                        }
                    }
                    else {
                         $child_ParentMAP.Add($leaf,@(@{$subSystemClassName = $subSystem.Name})+ $child_ParentMAP.Item($subSystem.Name))
                    }
               }
               $child_ParentMAP.Remove($subSystem.Name)
            }
            else{
                foreach($value in $subSystem.value.psObject.Properties.name){
                   #Below condition is to ignore DirectAttch Physical disks as they are not supported in CMC
                   #$child_ParentMAP will not add any entry for Physical disk coming under System->storage->Controller->controllerid
                   if(! ( ($rootParentName -eq "Controller") -and ($value -eq "PhysicalDisk") ) ) 
                   {
                       if(!($child_ParentMAP.Contains($value))){
                            if($array_SkipSubsystem.Contains($rootParentName)){
                                $subSystemClassName = "Dell.SDK." + $Device+"." +$rootParentName
                            }else{
                                $subSystemClassName = "Dell.SDK." + $Device +"." +$subSystem.Name +".Subsystem"
                            }
                       
                            if($component_ContainmentRelation.Contains($subSystem.Name)){
                                $child_ParentMAP.Add($value, @(@{$subSystemClassName = $rootkey}))
                            }
                            else{
                                $child_ParentMAP.Add($value, @(@{$subSystemClassName = $subSystem.Name}))
                            }
                       }
                   }
               }
               CreateSubsystemNodes $subSystem.value $subSystem.Name $Device

               if($array_SkipSubsystem.Contains($value)){
                    $child_ParentMAP.Remove($value)
               }
            }
        }
        if(($array_SkipSubsystem.Contains($subsystem.Name))  ){
            $child_ParentMAP.Remove($subsystem.Name)
	    }
    }
}

<# After creating basic structure i.e SCOM classes, Instances, Relationship below code is for
populating all instances with inventories which is available in Inventory.Json inventory creation start #>
Function CreateInventory()
{    
    param($jsonObj, $notAvailable)
     $IPAddress = $jsonObj.System.IPv4Address   

        foreach($Component in $jsonObj.PsObject.Properties) {
            if(!($skip_Components -contains $Component.Name)){
                psDebugLog -level 1 -message  ("Inventory started for component - " + $Component.Name)
                $cmpclass = "Dell.SDK." + $Device+"." + $Component.Name
                if($Component.Name -eq "StorageModule")
                {
                $cmpclass = "Dell.SDK." + $Device+"." + "ComputeModule"
                }
                $virtualDiskKey = ""
			    foreach($instance in $Component.Value) {
                    $createrelation = $false
                    $monitoringClass = GetMonitoringClass $cmpclass 
                    $classInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
                    $parentkeymap = $child_ParentMAP.Item($instance.key)
                    foreach($parent in $parentkeymap.keys){
                        if(!$array_SkipSubsystem.Contains($parentkeymap.$parent)){
                            

                            $parentClass = GetMonitoringClass $parent 

                            
                            if($parent -eq "Microsoft.Windows.Computer"){
                                $classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty("PrincipalName"),$parentkeymap.$parent)
                             }elseif(($parent -eq "Dell.SDK.CMC.VirtualDisk") -and ($Component.Name -eq "PhysicalDisk")){
                                $createrelation = $true
                                $virtualDiskKey = $parentkeymap.$parent
                             }else {
                            	$classInstance.SetMonitoringPropertyValue($parentClass.GetMonitoringProperty("Key"),$parentkeymap.$parent)
                            }
                        }
                    }
                    
                    foreach($attribute in $instance.PsObject.Properties) {
                        if($monitoringClass.PropertyCollection.Name.Contains($attribute.Name)){
                            $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($attribute.Name), $attribute.Value)
                        }
                    }
                    foreach($name in $monitoringClass.PropertyCollection.Name){ 
                        if(!($instance.PsObject.Properties.Name -contains $name)){
                            if(($Component.Name -eq "CMC") -and ($name -eq "Model")){
                                $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($name), $model)
                            }elseif(($Component.Name -eq "CMC") -and ($name -eq "FirmwareVersion")){
                                $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($name), $firmwareVersion)
                            }else {
                                $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($name), "Not Available")
                            }
                        }
                        if(($Component.Name -eq "CMC") -and ($jsonObj.System."$("_Type")" -eq "CMC") -and ($name -eq "State")){
                        $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty($name), "Primary")
                        }                                                 
                    }
                    if($monitoringClass.PropertyCollection.Name.Contains("Component")){
                        $classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Component"),$Component.Name)
                        
                    }
                    
                    if(($Component.Name -eq "ComputeModule") -or ($Component.Name -eq "StorageModule")) {$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.SlotName)}
                    elseif($Component.Name -eq "System"){$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $IPAddress)
					$classInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("IPAddress"), $IPAddress)}
                    elseif($Component.Name -eq "Slots_Summary"){$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), "Chassis Slots Summary")}
                    elseif($Component.Name -eq "Fan"){$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.ElementName)}
                    elseif($Component.Name -eq "CMC")
                    {
                        $state = 'CMC-Primary'
                        if(($instance.State -eq "Active") -or ($instance.State -eq "Standby"))
                        {
                            $state = "CMC-"+$instance.State
                        }
                        $classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $state)
                    }
                    elseif($Component.Name -eq "IOModule"){$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.Model)}
                    elseif($Component.Name -eq "PowerSupply"){$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.Name)}
                    elseif($Component.Name -eq "Enclosure"){$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.ProductName)}
                    elseif($Component.Name -eq "VirtualDisk"){$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.Name)}
                    else{$classInstance.SetMonitoringPropertyValue($entityClass.GetMonitoringProperty("DisplayName"), $instance.Key)}
                    $discoveryData.Add($classInstance)

                    <# If physical disk is also having relationship with virtual disk then add its value into hashmap $physicalDiskArray #>
                    if(($createrelation -eq  $true) -and ($Component.Name -eq "PhysicalDisk")){
                      if($physicalDiskArray.Keys -contains $virtualDiskKey){
                         $physicalDiskArray.Item($virtualDiskKey) = $physicalDiskArray.Item($virtualDiskKey) +@($classInstance)
                       }else{
                         $physicalDiskArray.Add($virtualDiskKey,@($classInstance))
                       }
                    } elseif($Component.Name -eq "VirtualDisk"){
                      $virtualDiskArray.Add($instance.Key, $classInstance)
                    }               
                    psDebugLog -level 1 -message  ($instance.Key + " instance discovered")
                }

                psDebugLog -level 1 -message  ("Inventory completed for component - " + $Component.Name)
		    }
            else{
                if($component.Name -eq "_deleted_attribs"){
                }
                elseif($Component.Name -ne "_deleted_comps"){
                }
                elseif($Component.Name -ne "_deleted_instances"){                       
                }
            }
        CreateContainmentForVirtualPhysical $virtualDiskArray $physicalDiskArray
        }
	
    }

Function CreateRootNode{
    param($rootParent, $comptTreeObj, $rootKey, $Device)
    $rootParentName = $comptTreeObj.Name
    $rootParentClassName = "Dell.SDK." + "$Device" + "." +$rootParentName
    $child_ParentMAP.Add($rootkey, @{$rootParentClassName=$rootKey} )

    $monitoringClass = GetMonitoringClass $rootParentClassName 
    $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($monitoringClass)
    $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
    $discoveryData.Add($rootInstance)

    if($model -match "M1000E"){
    $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($M1000ECMCClass)
    $CMCType = "M1000E"
    $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
    $discoveryData.Add($rootInstance)}
    elseif($model -match "MX7000"){
    $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($MX7000Class)
    $CMCType = "MX7000"
    $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
    $discoveryData.Add($rootInstance)}
    elseif($model -match "VRTX"){
    $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($VRTXCMCClass)
    $CMCType = "VRTX"
    $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
    $discoveryData.Add($rootInstance)}
    elseif($model -match "FX2"){
    $rootInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($FX2CMCClass)
    $CMCType = "FX2"
    $rootInstance.SetMonitoringPropertyValue($monitoringClass.GetMonitoringProperty("Key"),$rootkey)
    $discoveryData.Add($rootInstance)}
    else{psDebugLog -level 1 -message "CMC Model is not present in Json file "}

    <# Hash Map which contains different types of CMC and their class with root instance #>

    $hash_CMCTypesMap = @{"M1000E" = @("DellSDKM1000EGroup.Contains.M1000E","Dell.SDK.M1000EGroup", $rootInstance);
                          "MX7000" = @("DellSDKMX7000Group.Contains.MX7000","Dell.SDK.MX7000Group", $rootInstance);
                          "VRTX" = @("DellSDKVRTXGroup.Contains.VRTX","Dell.SDK.VRTXGroup", $rootInstance);
                          "FX2" = @("DellSDKFX2Group.Contains.FX2","Dell.SDK.FX2Group", $rootInstance)} 
    
    <# Creating the Containment relationship for CMC type #>

    CallCreateContainment $hash_CMCTypesMap.Item($CMCType)[1] $hash_CMCTypesMap.Item($CMCType)[2] $hash_CMCTypesMap.Item($CMCType)[0]
    
    if($rootInstance -ne $null){
    $targetComputer = $jsonObj.System."$("MachineName")"
    $healthServiceClassID = GetMonitoringClass "Microsoft.SystemCenter.HealthService"
    $healthServiceShudManageEntityClassID = GetMonitoringRelation "Microsoft.SystemCenter.HealthServiceShouldManageEntity"
    $healthServiceInstance = new-object Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringObject($healthServiceClassID) 
    $healthServiceInstance.SetMonitoringPropertyValue($computerClassID.GetMonitoringProperty("PrincipalName"),$targetComputer)
    $discoveryData.Add($healthServiceInstance)

    CreateContainmentRelation $healthServiceShudManageEntityClassID $healthServiceInstance $rootInstance
    
    CreateSubsystemNodes $rootParent $rootkey $Device
    }
}


<# Below loop will take care of each ServiceTag folder for chassis and it is responsible for
1. Creating Scom Class Instances 
2. Creating Relationship between Instances
3. Populating class instances with inventories
4.It uses ComTree.Json to get information of instances under different components and creates classes and instances.
5.It uses Inventory.Json to populate inventories to class instances.#>

$IPAddress = ""
$computerClassID = GetMonitoringClass "Microsoft.Windows.Computer"
$entityClass = GetMonitoringClass -Name "System.Entity"
#$serviceTagFolders = Get-ChildItem $nfspath
$array_SkipSubsystem = @("Controller", "Enclosure","ComputeModule","Slots_Summary","StorageModule")
$skip_Components = @("BladeSlot","KVM","License","Subsystem","_deleted_attribs", "_deleted_comps", "_deleted_instances","PeakTemperature","PowerConsumption","PowerUsageByDevice","TemperatureSensors","TemperatureMetrics")
$component_ContainmentRelation = @("IOModule","Storage","Fan","PCIDevice","PowerSupply","CMC")
$logLevel = 0
$discOverrides = Get-SCOMDiscovery -Name "Dell.CMC.Discovery" | Get-SCOMOverride -ErrorAction SilentlyContinue
if($discOverrides -ne $null){
    <#$discJsonObj = @()
    foreach($discOverride in $discOverrides){
        $discJsonObj += @(@{$discOverride.Parameter = $discOverride.Value})
    }
    $json = @{"DiscoveryOverrides"=@($discJsonObj)}
	$discOverrideJson = $json | ConvertTo-Json
    $response = Invoke-RestMethod 'http://example.com/api/people/1' -Method Put -Body $discOverrideJson -ContentType 'application/json'#>
    foreach ($override in $discOverrides){
        if($override.Parameter -eq "LogLevel"){
            $logLevel = $override.Value}
    }
}
foreach($serviceTag in $serviceTagFolders){
    if($logLevel -eq 1){	
        CreateLogFile $serviceTag + "_DellCMCInventory"}

    try{
        $jsonpath = $nfspath+"\"+$serviceTag+"\Inventory.json"
        if ((Get-Item $jsonpath -ErrorAction Ignore) -ne $null) {
	        $jsonObj = ConvertFrom-Json -InputObject (Get-Content $jsonpath -Raw)
        }

        $comptTreeFilePath = $nfspath+"\"+$serviceTag+"\CompTree.json"
        if ((Get-Item $comptTreeFilePath -ErrorAction Ignore) -ne $null) {
	        $comptTreeObj = ConvertFrom-Json -InputObject (Get-Content $comptTreeFilePath -Raw)
        }
    }
    catch{
        $ErrorMessage = $_.Exception.Message
        $FailedItem = $_.InvocationInfo.ScriptLineNumber
        psDebugLog -level 1 -message  ("Exception occurred : Line - " + $FailedItem + " - " + $ErrorMessage )
        $failed += $serviceTag.Name
        continue
    }

    $child_ParentMAP = @{}
    $rootParent = $comptTreeObj.System
    $comptTreeObj.PsOBject.Properties | ForEach-Object {if($_.Name -eq "System"){$comptTreeObj = $_}}
    $rootKey = $jsonObj.System."$("Key")"
    $Device = $jsonObj.System."$("_Type")"
	$urlString = $jsonObj.System."$("URLString")"
    $firmwareVersion = $jsonObj.System."$("MgmtControllerFirmwareVersion")"
    $model = $jsonObj.System."$("Model")"
    $virtualDiskArray = @{}
    $physicalDiskArray = @{}

    if($Device -match "NGM")
    {
        $Device = "CMC"

    }

    $targetComputer = $jsonObj.System."$("MachineName")"
    if($targetComputer -eq "Not Available")
    {
        $targetComputer = $computerName
    }

    If(($urlString -eq "Not Available") -or ($urlString -eq "") )
    {
        $IPAddress = $rootKey
    }else{
        $urlString = $urlString.Replace("https://","")
        $IPAddress = $urlString.Substring(0,$urlString.IndexOf(":"))
    }


    psDebugLog -level 1 -message  ("***** Discovery started for CMC *****")
    <# Call CreateRootNode method to create Root->Parent->Subcomponent hierarchy.OR
       To create all SCOM classes,relationship between these classes and all instances. 
    #>
    
    $script:jsondeleteinst = ConvertPSObjectToHashtable $jsonObj
    cleanUpInstances

    CreateRootNode $rootParent $comptTreeObj $rootKey $Device 
    

    <# After creating basic structure i.e SCOM classes, Instances, Relationship below code is for
    populating all instances with inventories which is available in Inventory.Json inventory creation start #>
    
    if($jsonObj -ne $null){
       CreateInventory $jsonObj
    }

    $discoveryData.Overwrite($Connector)
	psDebugLog -level 1 -message  ("***** Discovery ends for CMC *****")
}
